#include "processwidget.h"

#include <QPainterPath>
#include <QPainter>
#include <QPaintEvent>
#include <QStyleOption>
#include <QDebug>

#include "../style/usmproxystyle.h"
#include "../macro.h"
#include "../util.h"

ProcessWidget::ProcessWidget(QSettings *proSettings, QWidget *parent)
    : QWidget(parent)
    , m_proSettings(proSettings)
{
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->installEventFilter(this);
    initUI();
    initConnections();
    onSwitchProcType(m_nWhichNum);
}

ProcessWidget::~ProcessWidget()
{
    if (m_qtSettings) {
        delete m_qtSettings;
        m_qtSettings = nullptr;
    }
    if (m_ifSettings) {
        delete m_ifSettings;
        m_ifSettings = nullptr;
    }
}

void ProcessWidget::onWndClose()
{
    if (m_procTabView) {
        m_procTabView->onWndClose();
    }
}

void ProcessWidget::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QPainterPath path;

    QPainter painter(this);
    painter.setOpacity(1);
    painter.setRenderHint(QPainter::Antialiasing);  // 反锯齿;
    painter.setClipping(true);
    painter.setPen(Qt::transparent);

    path.addRect(this->rect());
    path.setFillRule(Qt::WindingFill);
    painter.setBrush(this->palette().base());
    painter.setPen(Qt::transparent);

    painter.drawPath(path);
    QStyleOption opt;
    opt.init(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &painter, this);
}

void ProcessWidget::initUI()
{
    // 初始化布局
    m_mainLayout = new QVBoxLayout();
    m_mainLayout->setContentsMargins(12,0,0,0);
    m_mainLayout->setSpacing(0);
    m_topBtnsLayout = new QHBoxLayout();
    m_topBtnsLayout->setContentsMargins(0,8,0,8);
    m_topBtnsLayout->setSpacing(0);
    m_topBtnsLayout->setAlignment(Qt::AlignHCenter);
    m_procTableLayout = new QVBoxLayout();
    m_procTableLayout->setContentsMargins(0,0,0,0);
    m_procTableLayout->setSpacing(0);

    // 初始化控件
    m_buttonGroup = new QButtonGroup(this);
    m_buttonGroup->setExclusive(true);
#if 1
    m_btnApplication = new KGroupButton();
    m_btnApplication->setCheckable(true);
    m_btnApplication->setAutoExclusive(true);
    m_btnApplication->setChecked(false);
    m_btnApplication->setPosition(GroupButtonStyleOption::Begin);
    m_btnApplication->setFixedSize(NORMALWIDTH,NORMALHEIGHT);
    m_buttonGroup->addButton(m_btnApplication, PROCESSTYPE_APPLICATION);
    m_topBtnsLayout->addWidget(m_btnApplication);
#else
    m_btnActive = new KGroupButton();
    m_btnActive->setCheckable(true);
    m_btnActive->setAutoExclusive(true);
    m_btnActive->setChecked(false);
    m_btnActive->setPosition(GroupButtonStyleOption::Begin);
    m_btnActive->setFixedSize(NORMALWIDTH,NORMALHEIGHT);
    m_buttonGroup->addButton(m_btnActive, PROCESSTYPE_ACTIVE);
    m_topBtnsLayout->addWidget(m_btnActive);
#endif
    m_btnBelongMe = new KGroupButton();
    m_btnBelongMe->setCheckable(true);
    m_btnBelongMe->setAutoExclusive(true);
    m_btnBelongMe->setChecked(false);
    m_btnBelongMe->setPosition(GroupButtonStyleOption::Middle);
    m_btnBelongMe->setFixedSize(NORMALWIDTH,NORMALHEIGHT);
    m_buttonGroup->addButton(m_btnBelongMe, PROCESSTYPE_BELONGME);
    m_topBtnsLayout->addWidget(m_btnBelongMe);

    m_btnAll = new KGroupButton();
    m_btnAll->setCheckable(true);
    m_btnAll->setAutoExclusive(true);
    m_btnAll->setChecked(false);
    m_btnAll->setPosition(GroupButtonStyleOption::End);
    m_btnAll->setFixedSize(NORMALWIDTH,NORMALHEIGHT);
    m_buttonGroup->addButton(m_btnAll, PROCESSTYPE_ALL);
    m_topBtnsLayout->addWidget(m_btnAll);

    m_topBtnsLayout->setSpacing(1);

    //初始化进程列表
    m_procTabView = new ProcessTableView(m_proSettings);
    m_procTabView->installEventFilter(this);
    m_procTableLayout->addWidget(m_procTabView);

    m_mainLayout->addLayout(m_topBtnsLayout);
    m_mainLayout->addLayout(m_procTableLayout);
    this->setLayout(m_mainLayout);
    initThemeMode();
}

void ProcessWidget::initConnections()
{
    connect(this,SIGNAL(changeProcessItemDialog(int)), m_procTabView, SLOT(onChangeProcessFilter(int)));
    connect(m_buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(onSwitchProcType(int)));
    connect(this, SIGNAL(searchSignal(QString)), m_procTabView, SLOT(onSearch(QString)), Qt::QueuedConnection);
}

void ProcessWidget::initThemeMode()
{
    const QByteArray idd(THEME_QT_SCHEMA);
    if(QGSettings::isSchemaInstalled(idd)) {
        m_qtSettings = new QGSettings(idd);
    }
    if (m_qtSettings) {
        //监听主题改变
        connect(m_qtSettings, &QGSettings::changed, this, [=](const QString &key)
        {
            if (key == "styleName") {
                qDebug()<<"style name changed";
                auto styleName = m_qtSettings->get("styleName").toString();

                if (styleName == "ukui-default" || styleName == "ukui-dark" || styleName == "ukui-white"
                        || styleName == "ukui-black" || styleName == "ukui-light" || styleName == "ukui") {
                    if (styleName == "ukui")
                        styleName = "ukui-default";
                    else if (styleName == "ukui-black")
                        styleName = "ukui-dark";
                    else if (styleName == "ukui-white")
                        styleName = "ukui-light";

                    if (m_btnApplication)
                        m_btnApplication->setStyle(new USMProxyStyle(styleName));
                    if (m_btnActive)
                        m_btnActive->setStyle(new USMProxyStyle(styleName));
                    if (m_btnBelongMe)
                        m_btnBelongMe->setStyle(new USMProxyStyle(styleName));
                    if (m_btnAll)
                        m_btnAll->setStyle(new USMProxyStyle(styleName));
                }
            } else if ("iconThemeName" == key) {
            } else if("systemFont" == key || "systemFontSize" == key) {
                qreal fontSize = m_qtSettings->get(FONT_SIZE).toString().toFloat();
                onThemeFontChanged(fontSize);
            }
        });
        auto styleName = m_qtSettings->get("styleName").toString();
        if (styleName == "ukui-default" || styleName == "ukui-dark" || styleName == "ukui-white"
                || styleName == "ukui-black" || styleName == "ukui-light" || styleName == "ukui") {
            if (styleName == "ukui")
                styleName = "ukui-default";
            else if (styleName == "ukui-black")
                styleName = "ukui-dark";
            else if (styleName == "ukui-white")
                styleName = "ukui-light";

            if (m_btnApplication)
                m_btnApplication->setStyle(new USMProxyStyle(styleName));
            if (m_btnActive)
                m_btnActive->setStyle(new USMProxyStyle(styleName));
            if (m_btnBelongMe)
                m_btnBelongMe->setStyle(new USMProxyStyle(styleName));
            if (m_btnAll)
                m_btnAll->setStyle(new USMProxyStyle(styleName));
        }
        qreal fontSize = m_qtSettings->get(FONT_SIZE).toString().toFloat();
        onThemeFontChanged(fontSize);
    }
    const QByteArray ifid(MENU_SCHEMA);
    if(QGSettings::isSchemaInstalled(ifid))
    {
        m_ifSettings = new QGSettings(ifid);
    }
    if (m_ifSettings && m_ifSettings->keys().contains(WHICH_MENU_KEY)) {
        auto whichNum = m_ifSettings->get(WHICH_MENU);
        if (whichNum.isValid()) {
            m_nWhichNum =  whichNum.toInt();
        }
    }
}

void ProcessWidget::onThemeFontChanged(float fFontSize)
{
    int nEditHeight = NORMALHEIGHT;
    if (fFontSize >= 14) {
        nEditHeight = NORMALHEIGHT+6;
    } else {
        nEditHeight = NORMALHEIGHT+3;
    }
    if (m_btnActive) {
        QString strContent = tr("Active Processes");
        if (!strContent.isEmpty()) {
            QString ShowValue = getElidedText(m_btnActive->font(), strContent, m_btnActive->width()-36);
            m_btnActive->setText(ShowValue);
            if (ShowValue != strContent) {
                m_btnActive->setToolTip(strContent);
            } else {
                m_btnActive->setToolTip("");
            }
        }
        m_btnActive->setFixedSize(NORMALWIDTH, nEditHeight);
    }
    if (m_btnApplication) {
        QString strContent = tr("Applications");
        if (!strContent.isEmpty()) {
            QString ShowValue = getElidedText(m_btnApplication->font(), strContent, m_btnApplication->width()-36);
            m_btnApplication->setText(ShowValue);
            if (ShowValue != strContent) {
                m_btnApplication->setToolTip(strContent);
            } else {
                m_btnApplication->setToolTip("");
            }
        }
        m_btnApplication->setFixedSize(NORMALWIDTH, nEditHeight);
    }
    if (m_btnBelongMe) {
        QString strContent = tr("My Processes");
        if (!strContent.isEmpty()) {
            QString ShowValue = getElidedText(m_btnBelongMe->font(), strContent, m_btnBelongMe->width()-36);
            m_btnBelongMe->setText(ShowValue);
            if (ShowValue != strContent) {
                m_btnBelongMe->setToolTip(strContent);
            } else {
                m_btnBelongMe->setToolTip("");
            }
        }
        m_btnBelongMe->setFixedSize(NORMALWIDTH, nEditHeight);
    }
    if (m_btnAll) {
        QString strContent = tr("All Process");
        if (!strContent.isEmpty()) {
            QString ShowValue = getElidedText(m_btnAll->font(), strContent, m_btnAll->width()-36);
            m_btnAll->setText(ShowValue);
            if (ShowValue != strContent) {
                m_btnAll->setToolTip(strContent);
            } else {
                m_btnAll->setToolTip("");
            }
        }
        m_btnAll->setFixedSize(NORMALWIDTH, nEditHeight);
    }
}

void ProcessWidget::onSwitchProcType(int nType)
{
#if 1
    if (nType == PROCESSTYPE_ACTIVE) {
        nType = PROCESSTYPE_APPLICATION;
    }
#endif
    QPushButton *typeButton = qobject_cast<QPushButton *>(m_buttonGroup->button(nType));
    if (typeButton) {
        typeButton->setChecked(true);
    }
    Q_EMIT changeProcessItemDialog(nType);
    if (m_ifSettings) {
        m_ifSettings->set(WHICH_MENU, nType);
    }
}
